/************************************************************************
*	KYBDBUF2.C - set up a larger ring buffer for the keyboard			*
*																		*
*	To compile: tcc kybdbuf2 mcb										*
************************************************************************/

#include <stdio.h>		/* printf(), puts()								*/
#include <stdlib.h>		/* exit(), EXIT_FAILURE							*/
#include <dos.h>		/* FP_SEG(), geninterrupt(), MK_FP()			*/
#include <assert.h>		/* assert()										*/
#include "mcb.h"		/* Memory control block definitions				*/

#define MAXSIZE 16		/* Maximum buffer size: 128 keystrokes			*/
#define MINSIZE 4		/* Minimum buffer size:  32 keystrokes			*/
#define FREE	(0x00)	/* Free block owner tag							*/
#define TRUE	(1==1)
#define FALSE	(!TRUE)


void main(void)
	{
	int found;
	unsigned int i, far *p;
	unsigned int far *HeadOff, far *TailOff,
				 far *BegOff,  far *EndOff;
	MCB huge *MCBptr;

	assert(sizeof(MCB)==mcbSIZE);
	puts("KYBDBUF2 - extend the size of the keyboard buffer");

	/* Free the environment block */
	p = MK_FP(_psp,0);
	printf("Environment block address: %04x:0000, length: %d\n",
			p[22]-1,((MCB *)MK_FP(p[22]-1,0))->size*16);
	_AX = 0x4900;
	_ES = p[22];
	geninterrupt(0x21);

	/* Try to find an MCB large enough for the keyboard buffer	*/
	for (found=FALSE, MCBptr=getFirstMCB();
		 !found && MCBptr->marker=='M'; MCBptr=getNextMCB(MCBptr))
		{
		if (MCBptr->owner != FREE)		/* Examine only free RAM*/
			continue;
		if (MCBptr->size > MAXSIZE)		/* Don't waste HUGE blks*/
			continue;
		if (MCBptr->size < MINSIZE)		/* Don't use tiny blocks*/
			continue;
		found = TRUE;
		printf("Block %Fp holds %d bytes\n",
					MCBptr, MCBptr->size*16);
		break;
		}


	/* Did we find a suitable free block? */
	if ( !found )
		{
		puts("No suitable free RAM for new keyboard buffer");
		exit(EXIT_FAILURE);
		}

	/* Is the block we found close enough to segment 0x40?	*/
	if (FP_SEG(MCBptr)-0x40U > 0x1000UL-MCBptr->size*16)
		{
		puts("No suitable RAM close enough to BIOS data area");
		exit(EXIT_FAILURE);
		}

	/* First, lay claim to the block of RAM	*/
	MCBptr->owner = 0xffff;

	/* Next, prepare the pointers to the keyboard pointers */
	HeadOff = MK_FP(0x40,0x1a);
	TailOff = MK_FP(0x40,0x1c);
	BegOff  = MK_FP(0x40,0x80);
	EndOff  = MK_FP(0x40,0x82);

	/* Finally, we can set up the appropriate keyboard pointers	*/
	printf("Our keyboard buffer is from 0040:%04x to 0040:%04x\n",
			((unsigned)FP_SEG(MCBptr)-0x3f)*16,
			((unsigned)FP_SEG(MCBptr)-0x3f+MCBptr->size)*16);

	disable();
	*HeadOff=*TailOff=*BegOff=((unsigned)FP_SEG(MCBptr)-0x3f)*16;
	*EndOff=((unsigned)FP_SEG(MCBptr)-0x3f+MCBptr->size)*16;
	enable();
	}
